import requests
from requests.auth import HTTPBasicAuth
import urllib3
import inspect
from abc import ABCMeta, abstractmethod


class APIPlugin(metaclass=ABCMeta):
    def __init__(self, verify: bool = False, warnings: bool = False, api_version: [str, bytes] = 'v1'):
        """
        The APIPlugin is the foundation to creating API Plugins from many different vendors. The APIPlugin help you
        create the most dynamic API client possible with minimal coding in few different ways. The APIPlugin is design
        to work with the RESTful API designs most used by developers specially in the Network Field.
        :param verify: SSL Cert verification, default to False to not verify SSL Cert.
        :param warnings: Request Warnings, default to False to disable all warnings.
        :param api_version: The version of the API as a string, defaults to v1.
        """
        self.verify = bool(verify)
        self.api_version = api_version
        if warnings is False:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    @property
    @abstractmethod
    def headers(self):
        """
        The HTTP Headers as a python dictionary
        :return:
        """
        pass

    @property
    @abstractmethod
    def base_url(self):
        """
        The Base URL will be the namespace of the URL that will be the same across all the API calls.
        :return:
        """
        pass

    @abstractmethod
    def connect(self, url: [str, None] = None, username: [str, bytes] = '',
                password: [str, bytes] = '') -> requests.Response:
        """
        The way to connect to the API by authenticating and generating tokens.
        :param url: The URL of the resource you are trying to access.
        :param username: The username of the system you are trying to authenticate.
        :param password: The password of the system you are trying to authenticate.
        :return:
        """
        pass

    @abstractmethod
    def disconnect(self):
        """
        The way to disconnect from the API by revoking tokens or logging out of the session.
        :return:
        """
        pass

    def get(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
            **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)

    def post(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
             **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)

    def put(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
            **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)

    def delete(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
               **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)



import FMCAPIClient
import unittest
import os
import json

url = 'https://FMC-server.local'
username = os.environ.get('USERNAME')
password = os.environ.get('PASSWORD')


def run_full_test():
    client = FMCAPIClient.Client()

    print(f'***** Testing: Login '.ljust(60, '*'))
    client.connect(url, username, password)

    print(f'***** Testing: GET method '.ljust(60, '*'))
    response = client.get(method=f'/devicegroups/devicegrouprecords')
    print(json.dumps(response.json(), indent=4))

    response = client.post(method='/devicegroups/devicegrouprecords', data={'name': 'test_group',
                                                                            'type': 'DeviceGroup'})
    print(json.dumps(response.json(), indent=4))
    group_id = response.json()['id']

    response = client.put(method=f'/devicegroups/devicegrouprecords/{group_id}', data={'id': group_id,
                                                                                       'name': 'test_group_updated',
                                                                                       'type': 'DeviceGroup'})
    print(json.dumps(response.json(), indent=4))

    response = client.delete(method=f'/devicegroups/devicegrouprecords/{group_id}')
    print(json.dumps(response.json(), indent=4))

    print(f'***** Testing: Logout '.ljust(60, '*'))
    client.disconnect()


class TestFMCAPIWrapper(unittest.TestCase):

    def test_authentication(self):
        client = FMCAPIClient.Client()

        response = client.connect(url, username, password)
        self.assertEqual(response.status_code, 204)

        response = client.refresh_token_func()
        self.assertEqual(response.status_code, 204)

        client.disconnect()
        self.assertEqual(response.status_code, 204)

    def test_methods_get(self):
        client = FMCAPIClient.Client()

        client.connect(url, username, password)

        response = client.get(method=f'/devicegroups/devicegrouprecords')
        self.assertEqual(response.status_code, 200)

        response = client.post(method='/devicegroups/devicegrouprecords', data={'name': 'test_group',
                                                                                'type': 'DeviceGroup'})
        self.assertEqual(response.status_code, 201)
        group_id = response.json()['id']

        response = client.put(method=f'/devicegroups/devicegrouprecords/{group_id}', data={'id': group_id,
                                                                                           'name': 'test_group_updated',
                                                                                           'type': 'DeviceGroup'})
        self.assertEqual(response.status_code, 200)

        response = client.delete(method=f'/devicegroups/devicegrouprecords/{group_id}')
        self.assertEqual(response.status_code, 200)

        client.disconnect()


if __name__ == '__main__':
    unittest.main()



from FMCAPIClient.api_interface import APIPlugin
import requests
from requests.auth import HTTPBasicAuth
from requests.models import Response
from datetime import datetime, timedelta
import inspect


class Client(APIPlugin):
    headers = {'Content-Type': 'application/json'}
    base_url = None
    token = None
    auth = None
    token_expire = datetime.now()
    token_refresh = None
    token_refresh_count = 0
    domain_uuid = None
    server = None
    token_expiration_period = 29

    def connect(self, url: [str, bytes] = '', username: [str, bytes] = '', password: [str, bytes] = ''):
        self.server = url.strip("/")
        self.auth = HTTPBasicAuth(username, password)
        response = requests.post(f'{self.server}/api/fmc_platform/{self.api_version}/auth/generatetoken',
                                 headers=self.headers, auth=self.auth, verify=self.verify)
        if response.status_code == 204:
            self.domain_uuid = response.headers.get('DOMAIN_UUID')
            self.token = response.headers.get('X-auth-access-token')
            self.token_refresh = response.headers.get('X-auth-refresh-token')
            self.token_refresh_count = 0
            if self.token_expiration_period > 30 or self.token_expiration_period < 1:
                self.token_expiration_period = 30
            self.token_expire = datetime.now() + timedelta(minutes=self.token_expiration_period)
            self.headers.update({'X-auth-access-token': self.token})

        self.base_url = f'{self.server}/api/fmc_config/{self.api_version}/domain/{self.domain_uuid}'
        return response

    def disconnect(self):
        response = requests.post(f'{self.server}/api/fmc_platform/{self.api_version}/auth/revokeaccess',
                                 headers=self.headers, verify=self.verify)
        return response

    def refresh_token_func(self):
        response = requests.post(f'{self.server}/api/fmc_platform/{self.api_version}/auth/refreshtoken',
                                 headers={'Content-Type': 'application/json',
                                          'X-auth-access-token': self.token,
                                          'X-auth-refresh-token': self.token_refresh},
                                 verify=self.verify)

        if response.status_code == 204:
            self.domain_uuid = response.headers.get('DOMAIN_UUID')
            self.token = response.headers.get('X-auth-access-token')
            self.token_refresh = response.headers.get('X-auth-refresh-token')
            self.token_refresh_count += 1
            if self.token_expiration_period > 30 or self.token_expiration_period < 1:
                self.token_expiration_period = 30
            self.token_expire = datetime.now() + timedelta(minutes=self.token_expiration_period)
            self.headers.update({'X-auth-access-token': self.token})

        return response

    def validate_token(self):
        """This validate token is only used internal by the class to check the token in memory with
        the token information it has in memory. Is not design check any external tokens."""
        if self.token_refresh_count >= 3:
            response = self.connect(self.server, self.auth.username, self.auth.password)
        elif self.token_expire <= datetime.now():
            response = self.refresh_token_func()
        else:
            response = Response()
            response.status_code = 204

        return response

    def get(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
            **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        self.validate_token()
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)

    def post(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
             **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        self.validate_token()
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)

    def put(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
            **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        self.validate_token()
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)

    def delete(self, url: [str, None] = None, method: [str, bytes] = '', data: dict = None, auth: HTTPBasicAuth = None,
               **kwargs) -> requests.Response:
        """

        :param url: The URL of the resource you are trying to access. Defaults to base_url + method.
        :param method: The method is the part of the url that changes and is appended to basic_url.
        :param data: The data to change or update as a Python dictionary. Defaults to None.
        :param auth: A Requests HTTPBasicAuth with the username and password. Defaults to None.
        :param kwargs: All parameters required by the API call.
        :return: The requests.Response of the API call.
        """
        self.validate_token()
        http_method = inspect.stack()[0][3].upper()
        url = f'{self.base_url}/{method.strip("/")}' if url is None else url
        return requests.request(http_method, url, headers=self.headers, verify=self.verify, json=data,
                                auth=auth, params=kwargs)



from setuptools import setup

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='FMCAPIClient',
    packages=['FMCAPIClient'],
    version='0.0.2',
    license='MIT',
    description='An API Client to use FMC RESTful API.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Victor M Santiago',
    author_email='vsantiago113sec@gmail.com',
    url='https://github.com/vsantiago113/FMCAPIClient',
    download_url='https://github.com/vsantiago113/FMCAPIClient/archive/0.0.2.tar.gz',
    keywords=['FMC'],
    python_requires='>=3.4.0',
    install_requires=[
        'requests',
        'urllib3'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
